@@ -5,6 +5,7 @@ use strict;
 use warnings;
 use Crypt::OpenSSL::X509();
 use Crypt::OpenSSL::RSA();
+use Crypt::OpenSSL::PKCS10();
 use Convert::ASN1();
 use Digest();
 use MIME::Base64();
@@ -21,7 +22,7 @@ our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK, );
 sub _CERTIFICATE_HEADER_SIZE { return 40; }
 sub _MAX_PUBLIC_KEY_SIZE     { return 65_536; }
 
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 
 sub pin_sha256 {
     my ($path) = @_;
@@ -42,6 +43,13 @@ sub pin_sha256 {
         $pem_encoded_public_key_string =
           _process_pem_public_key( $handle, $file_header, $path );
     }
+    elsif ( $file_header =~
+        /^[-]{5}BEGIN[ ](?:NEW[ ])?CERTIFICATE[ ]REQUEST[-]{5}/smx )
+    {
+        $pem_encoded_public_key_string =
+          _process_pem_pkcs10_certificate_request( $handle, $file_header,
+            $path );
+    }
     else {
         $pem_encoded_public_key_string =
           _check_for_der_encoded_x509_certificate( $handle, $file_header,
@@ -91,6 +99,13 @@ s/^([-]{5}BEGIN[ ])(?:X[.]?509|TRUSTED)[ ](CERTIFICATE[-]{5})/$1$2/smx;
     return $pem_encoded_public_key_string;
 }
 
+sub _process_pem_pkcs10_certificate_request {
+    my ( $handle, $file_header, $path ) = @_;
+    my $req = Crypt::OpenSSL::PKCS10->new_from_file($path);
+    my $pem_encoded_public_key_string = $req->get_pem_pubkey();
+    return $pem_encoded_public_key_string;
+}
+
 sub _check_for_der_encoded_x509_certificate {
     my ( $handle, $file_header, $path ) = @_;
     my $pem_encoded_public_key_string;
@@ -180,7 +195,7 @@ HTTP::PublicKeyPins - Generate RFC 7469 HTTP Public Key Pin (HPKP) header values
 
 =head1 VERSION
 
-Version 0.10
+Version 0.11
 
 =head1 SYNOPSIS
 
@@ -195,7 +210,7 @@ Make it more difficult for the bad guys to Man-In-The-Middle your users TLS sess
             'pin-sha256="'
           . pin_sha256('/etc/pki/tls/certs/example.pem')
           . '"; pin-sha256="'
-          . pin_sha256('/etc/pki/tls/certs/backup.pem')
+          . pin_sha256('/etc/pki/tls/certs/backup.req')
           . '"; report-uri="https://example.com/pkp-report.pl' );
 
 
@@ -207,7 +222,7 @@ This module allows the calculation of RFC 7469 HTTP Public Key Pin header values
 
 =head2 pin_sha256
 
-This function accepts the path to a L<X.509 Certificate|http://tools.ietf.org/html/rfc5280>.  It will load the public key from the certificate and prepare the appropriate value for the pin_sha256 parameter of the Public-Key-Pins value. This function will also make an attempt to read public keys (in PEM (L<SubjectPublicKeyInfo|http://tools.ietf.org/html/rfc5280#section-4.1.2.7> or L<PKCS#1|https://tools.ietf.org/html/rfc344>) or DER format).
+This function accepts the path to a L<X.509 Certificate|http://tools.ietf.org/html/rfc5280>.  It will load the public key from the certificate and prepare the appropriate value for the pin_sha256 parameter of the Public-Key-Pins value. This function will also make an attempt to read public keys (in PEM (L<SubjectPublicKeyInfo|http://tools.ietf.org/html/rfc5280#section-4.1.2.7> or L<PKCS#1|https://tools.ietf.org/html/rfc2437>) or DER format) and L<PKCS#10 Certificate Requests|https://tools.ietf.org/html/rfc2986> in PEM or DER format.
 
 =head1 SUBROUTINES/METHODS
 
@@ -219,19 +234,27 @@ None.  This module only has the one exported function.
  
 =item C<< Failed to open %s for reading >>
  
-Failed to open the supplied X.509 Certificate or Public Key file
+Failed to open the supplied X.509 Certificate, PKCS10 Certificate Request or Public Key file
  
 =item C<< Failed to read from %s >>
  
-Failed to read from the X.509 Certificate or Public Key file
+Failed to read from the X.509 Certificate, PKCS10 Certificate Request or Public Key file
 
-=item C<< %s is not an X.509 Certificate or Public Key >>
+=item C<< %s is not an X.509 Certificate, PKCS10 Certificate Request or Public Key >>
  
-The supplied input file does not look like X.509 Certificate File or Public Key. The X.509 Certificate file may be encoded in PEM or DER format. A PEM encoded X.509 Certificate file has the following header
+The supplied input file does not look like X.509 Certificate File, PKCS10 Certificate Request or Public Key. These files may be encoded in PEM or DER format. A PEM encoded X.509 Certificate file has the following header
 
   -----BEGIN CERTIFICATE-----
 
-A PEM encoded Public Key has the following header
+A PEM encoded PKCS#10 Certificate Request has the following header
+
+  -----BEGIN CERTIFICATE REQUEST-----
+
+A PEM encoded PKCS#1 Public Key has the following header
+
+  -----BEGIN RSA PUBLIC KEY-----
+
+A PEM encoded SubjectPublicKeyInfo Public Key has the following header
 
   -----BEGIN PUBLIC KEY-----
 
@@ -246,8 +269,9 @@ HTTP::PublicKeyPins requires no configuration files or environment variables.
 HTTP::PublicKeyPins requires the following non-core modules
  
   Convert::ASN1
-  Crypt::OpenSSL::X509
+  Crypt::OpenSSL::PKCS10
   Crypt::OpenSSL::RSA
+  Crypt::OpenSSL::X509
   Digest
  
 =head1 INCOMPATIBILITIES
@@ -262,7 +286,9 @@ None known.
 
 =item L<X.509 Certificate|http://tools.ietf.org/html/rfc5280>
 
-=item L<PKCS#1|https://tools.ietf.org/html/rfc344>
+=item L<PKCS#1|https://tools.ietf.org/html/rfc2437>
+
+=item L<PKCS#10|https://tools.ietf.org/html/rfc2986>
 
 =back
 
@@ -283,7 +309,6 @@ You can find documentation for this module with the perldoc command.
 
     perldoc HTTP::PublicKeyPins
 
-
 You can also look for information at:
 
 =over 4
